@@ -1,5 +1,5 @@
-/* $XConsortium: imake.c /main/88 1995/12/08 16:33:13 gildea $ */
-/* $XFree86: xc/config/imake/imake.c,v 3.11 1996/05/06 05:52:13 dawes Exp $ */
+/* $XConsortium: imake.c /main/90 1996/11/13 14:43:23 lehors $ */
+/* $XFree86: xc/config/imake/imake.c,v 3.13.2.1 1997/05/11 05:04:06 dawes Exp $ */
 
 /***************************************************************************
  *                                                                         *
@@ -89,21 +89,52 @@ in this Software without prior written authorization from the X Consortium.
  *	or the default, Imake.tmpl.  Call this <template>
  *   3. Determine the name of the imakeCfile from the command line (-C)
  *	or the default, Imakefile.c.  Call this <imakeCfile>
- *   3. Store three lines of input into <imakeCfile>:
- *		#define IMAKE_TEMPLATE		" <template> "
+ *   4. Store lines of input into <imakeCfile>:
+ *	- A c-style comment header (see ImakefileCHeader below), used
+ *	  to recognize temporary files generated by imake.
+ *	- If DEFAULT_OS_NAME is defined, format the utsname struct and
+ *	  call the result <defaultOsName>.  Add:
+ *		#define DefaultOSName <defaultOsName>
+ *	- If DEFAULT_OS_MAJOR_REV is defined, format the utsname struct
+ *	  and call the result <defaultOsMajorVersion>.  Add:
+ *		#define DefaultOSMajorVersion <defaultOsMajorVersion>
+ *	- If DEFAULT_OS_MINOR_REV is defined, format the utsname struct
+ *	  and call the result <defaultOsMinorVersion>.  Add:
+ *		#define DefaultOSMinorVersion <defaultOsMinorVersion>
+ *	- If DEFAULT_OS_TEENY_REV is defined, format the utsname struct
+ *	  and call the result <defaultOsTeenyVersion>.  Add:
+ *		#define DefaultOSTeenyVersion <defaultOsTeenyVersion>
+ *	- If the file "localdefines" is readable in the current
+ *	  directory, print a warning message to stderr and add: 
+ *		#define IMAKE_LOCAL_DEFINES	"localdefines"
+ *		#include IMAKE_LOCAL_DEFINES
+ *	- If the file "admindefines" is readable in the current
+ *	  directory, print a warning message to stderr and add: 
+ *		#define IMAKE_ADMIN_DEFINES	"admindefines"
+ *		#include IMAKE_ADMIN_DEFINES
+ *	- The following lines:
  *		#define INCLUDE_IMAKEFILE	< <imakefile> >
+ *		#define IMAKE_TEMPLATE		" <template> "
  *		#include IMAKE_TEMPLATE
- *	Start up cpp and provide it with this file.
+ *	- If the file "adminmacros" is readable in the current
+ *	  directory, print a warning message to stderr and add: 
+ *		#define IMAKE_ADMIN_MACROS	"adminmacros"
+ *		#include IMAKE_ADMIN_MACROS
+ *	- If the file "localmacros" is readable in the current
+ *	  directory, print a warning message to stderr and add: 
+ *		#define IMAKE_LOCAL_MACROS	"localmacros"
+ *		#include IMAKE_LOCAL_MACROS
+ *   5. Start up cpp and provide it with this file.
  *	Note that the define for INCLUDE_IMAKEFILE is intended for
  *	use in the template file.  This implies that the imake is
  *	useless unless the template file contains at least the line
  *		#include INCLUDE_IMAKEFILE
- *   4. Gather the output from cpp, and clean it up, expanding @@ to
+ *   6. Gather the output from cpp, and clean it up, expanding @@ to
  *	newlines, stripping trailing white space, cpp control lines,
  *	and extra blank lines, and changing XCOMM to #.  This cleaned
  *	output is placed in a new file, default "Makefile", but can
  *	be specified with -s or -e options.
- *   5. Optionally start up make on the resulting file.
+ *   7. Optionally start up make on the resulting file.
  *
  * The design of the template makefile should therefore be:
  *	<set global macros like CFLAGS, etc.>
@@ -118,72 +149,76 @@ in this Software without prior written authorization from the X Consortium.
 #endif
 #include <ctype.h>
 #ifdef WIN32
-#include "Xw32defs.h"
+# include "Xw32defs.h"
 #endif
 #ifndef X_NOT_POSIX
-#ifndef _POSIX_SOURCE
-#define _POSIX_SOURCE
-#endif
+# ifndef _POSIX_SOURCE
+#  define _POSIX_SOURCE
+# endif
 #endif
 #include <sys/types.h>
 #include <fcntl.h>
 #ifdef X_NOT_POSIX
-#ifndef WIN32
-#include <sys/file.h>
-#endif
+# ifndef WIN32
+#  include <sys/file.h>
+# endif
 #else
-#include <unistd.h>
+# include <unistd.h>
 #endif
 #ifdef ISC
-#include <unistd.h>
+# include <unistd.h>
 #endif
 #if defined(X_NOT_POSIX) || defined(_POSIX_SOURCE)
-#include <signal.h>
+# include <signal.h>
 #else
-#define _POSIX_SOURCE
-#include <signal.h>
-#undef _POSIX_SOURCE
+# define _POSIX_SOURCE
+# include <signal.h>
+# undef _POSIX_SOURCE
 #endif
 #if !defined(SIGCHLD) && defined(SIGCLD)
-#define SIGCHLD		SIGCLD
+# define SIGCHLD		SIGCLD
 #endif
 #include <sys/stat.h>
 #ifndef X_NOT_POSIX
-#ifdef _POSIX_SOURCE
-#include <sys/wait.h>
-#else
-#define _POSIX_SOURCE
-#include <sys/wait.h>
-#undef _POSIX_SOURCE
-#endif
-#define waitCode(w)	WEXITSTATUS(w)
-#define waitSig(w)	WTERMSIG(w)
+# ifdef _POSIX_SOURCE
+#  ifdef SCO325
+#   include <sys/procset.h>
+#   include <sys/siginfo.h>
+#  endif
+#  include <sys/wait.h>
+# else
+#  define _POSIX_SOURCE
+#  include <sys/wait.h>
+#  undef _POSIX_SOURCE
+# endif
+# define waitCode(w)	WEXITSTATUS(w)
+# define waitSig(w)	WTERMSIG(w)
 typedef int		waitType;
 #else /* X_NOT_POSIX */
-#ifdef SYSV
-#define waitCode(w)	(((w) >> 8) & 0x7f)
-#define waitSig(w)	((w) & 0xff)
+# ifdef SYSV
+#  define waitCode(w)	(((w) >> 8) & 0x7f)
+#  define waitSig(w)	((w) & 0xff)
 typedef int		waitType;
-#else /* SYSV */
-#ifdef WIN32
-#include <process.h>
+# else /* SYSV */
+#  ifdef WIN32
+#   include <process.h>
 typedef int		waitType;
-#else
-#include <sys/wait.h>
-#define waitCode(w)	((w).w_T.w_Retcode)
-#define waitSig(w)	((w).w_T.w_Termsig)
+#  else
+#   include <sys/wait.h>
+#   define waitCode(w)	((w).w_T.w_Retcode)
+#   define waitSig(w)	((w).w_T.w_Termsig)
 typedef union wait	waitType;
-#endif
-#endif
-#ifndef WIFSIGNALED
-#define WIFSIGNALED(w) waitSig(w)
-#endif
-#ifndef WIFEXITED
-#define WIFEXITED(w) waitCode(w)
-#endif
+#  endif
+# endif
+# ifndef WIFSIGNALED
+#  define WIFSIGNALED(w) waitSig(w)
+# endif
+# ifndef WIFEXITED
+#  define WIFEXITED(w) waitCode(w)
+# endif
 #endif /* X_NOT_POSIX */
 #ifndef X_NOT_STDC_ENV
-#include <stdlib.h>
+# include <stdlib.h>
 #else
 char *malloc(), *realloc();
 void exit();
@@ -198,8 +233,6 @@ extern char	*getenv();
 #ifdef X_NOT_STDC_ENV
 extern int	errno;
 #endif
-#include "imakemdep.h"
-
 #ifdef __minix_vmd
 #define USE_FREOPEN		1
 #endif
@@ -210,17 +243,29 @@ extern int	errno;
 #ifdef __EMX__
 #define USE_STRERROR		1
 #endif
+#ifndef WIN32
+#include <sys/utsname.h>
+#endif
+#ifndef SYS_NMLN
+# ifdef _SYS_NMLN
+#  define SYS_NMLN _SYS_NMLN
+# else
+#  define SYS_NMLN 257
+# endif
+#endif
+#include "imakemdep.h"
+
 /*
  * This define of strerror is copied from (and should be identical to)
  * Xos.h, which we don't want to include here for bootstrapping reasons.
  */
 #ifndef USE_STRERROR
-#ifndef strerror
+# ifndef strerror
 extern char *sys_errlist[];
 extern int sys_nerr;
-#define strerror(n) \
+#  define strerror(n) \
     (((n) >= 0 && (n) < sys_nerr) ? sys_errlist[n] : "unknown error")
-#endif
+# endif
 #endif
 
 #define	TRUE		1
@@ -228,9 +273,9 @@ extern int sys_nerr;
 
 #ifdef FIXUP_CPP_WHITESPACE
 int	InRule = FALSE;
-#ifdef INLINE_SYNTAX
+# ifdef INLINE_SYNTAX
 int	InInline = 0;
-#endif
+# endif
 #endif
 #ifdef MAGIC_MAKE_VARS
 int xvariable = 0;
@@ -245,24 +290,24 @@ int xvariables[10];
 #ifdef FIXUP_CPP_WHITESPACE
 void KludgeOutputLine(), KludgeResetRule();
 #else
-#define KludgeOutputLine(arg)
-#define KludgeResetRule()
+# define KludgeOutputLine(arg)
+# define KludgeResetRule()
 #endif
 
 typedef	unsigned char	boolean;
 
 #ifdef USE_CC_E
-#ifndef DEFAULT_CC
-#define DEFAULT_CC "cc"
-#endif
+# ifndef DEFAULT_CC
+#  define DEFAULT_CC "cc"
+# endif
 #else
-#ifndef DEFAULT_CPP
-#ifdef CPP_PROGRAM
-#define DEFAULT_CPP CPP_PROGRAM
-#else
-#define DEFAULT_CPP "/lib/cpp"
-#endif
-#endif
+# ifndef DEFAULT_CPP
+#  ifdef CPP_PROGRAM
+#   define DEFAULT_CPP CPP_PROGRAM
+#  else
+#   define DEFAULT_CPP "/lib/cpp"
+#  endif
+# endif
 #endif
 
 char *cpp = NULL;
@@ -291,11 +336,25 @@ char	*ReadLine();
 char	*CleanCppInput();
 char	*Strdup();
 char	*Emalloc();
-void	LogFatalI(), LogFatal();
+void	LogFatalI(), LogFatal(), LogMsg();
+
+void	showit();
+void	wrapup();
+void	init();
+void	AddMakeArg();
+void	AddCppArg();
+void	SetOpts();
+void	CheckImakefileC();
+void	cppit();
+void	makeit();
+void	CleanCppOutput();
+boolean	isempty();
+void	writetmpfile();
 
 boolean	verbose = FALSE;
 boolean	show = TRUE;
 
+int
 main(argc, argv)
 	int	argc;
 	char	**argv;
@@ -338,6 +397,7 @@ main(argc, argv)
 	exit(0);
 }
 
+void
 showit(fd)
 	FILE	*fd;
 {
@@ -351,14 +411,15 @@ showit(fd)
 	    LogFatal("Cannot read %s.", tmpMakefile);
 }
 
+void
 wrapup()
 {
 	if (tmpMakefile != Makefile)
 		unlink(tmpMakefile);
 	if (cleanedImakefile && cleanedImakefile != Imakefile)
-	    unlink(cleanedImakefile);
+		unlink(cleanedImakefile);
 	if (haveImakefileC)
-	    unlink(ImakefileC);
+		unlink(ImakefileC);
 }
 
 #ifdef SIGNALRETURNSINT
@@ -376,6 +437,7 @@ catch(sig)
 /*
  * Initialize some variables.
  */
+void
 init()
 {
 	register char	*p;
@@ -394,7 +456,7 @@ init()
 	 */
 	if (p = getenv("IMAKEINCLUDE")) {
 		if (*p != '-' || *(p+1) != 'I')
-			LogFatal("Environment var IMAKEINCLUDE %s\n",
+			LogFatal("Environment var IMAKEINCLUDE %s",
 				"must begin with -I");
 		AddCppArg(p);
 		for (; *p; p++)
@@ -415,6 +477,7 @@ init()
 #endif
 }
 
+void
 AddMakeArg(arg)
 	char	*arg;
 {
@@ -425,6 +488,7 @@ AddMakeArg(arg)
 	make_argv[ make_argindex ] = NULL;
 }
 
+void
 AddCppArg(arg)
 	char	*arg;
 {
@@ -435,6 +499,7 @@ AddCppArg(arg)
 	cpp_argv[ cpp_argindex ] = NULL;
 }
 
+void
 SetOpts(argc, argv)
 	int	argc;
 	char	**argv;
@@ -458,7 +523,7 @@ SetOpts(argc, argv)
 		    else {
 			argc--, argv++;
 			if (! argc)
-			    LogFatal("No description arg after -f flag\n", "");
+			    LogFatal("No description arg after -f flag", "");
 			Imakefile = argv[0];
 		    }
 		} else if (argv[0][1] == 's') {
@@ -468,7 +533,7 @@ SetOpts(argc, argv)
 		    else {
 			argc--, argv++;
 			if (!argc)
-			    LogFatal("No description arg after -s flag\n", "");
+			    LogFatal("No description arg after -s flag", "");
 			Makefile = ((argv[0][0] == '-') && !argv[0][1]) ?
 			    NULL : argv[0];
 		    }
@@ -482,7 +547,7 @@ SetOpts(argc, argv)
 		    else {
 			argc--, argv++;
 			if (! argc)
-			    LogFatal("No description arg after -T flag\n", "");
+			    LogFatal("No description arg after -T flag", "");
 			Template = argv[0];
 		    }
 		} else if (argv[0][1] == 'C') {
@@ -491,7 +556,7 @@ SetOpts(argc, argv)
 		    else {
 			argc--, argv++;
 			if (! argc)
-			    LogFatal("No imakeCfile arg after -C flag\n", "");
+			    LogFatal("No imakeCfile arg after -C flag", "");
 			ImakefileC = argv[0];
 		    }
 		} else if (argv[0][1] == 'v') {
@@ -515,7 +580,8 @@ SetOpts(argc, argv)
 	AddCppArg(ImakefileC);
 }
 
-char *FindImakefile(Imakefile)
+char *
+FindImakefile(Imakefile)
 	char	*Imakefile;
 {
 	if (Imakefile) {
@@ -552,15 +618,28 @@ LogFatal(x0,x1)
 		return;
 	entered = TRUE;
 
-	fprintf(stderr, "%s: ", program);
-	if (errno)
-		fprintf(stderr, "%s: ", strerror(errno));
-	fprintf(stderr, x0,x1);
+	LogMsg(x0, x1);
 	fprintf(stderr, "  Stop.\n");
 	wrapup();
 	exit(1);
 }
 
+void
+LogMsg(x0,x1)
+	char *x0, *x1;
+{
+	int error_number = errno;
+
+	if (error_number) {
+		fprintf(stderr, "%s: ", program);
+		fprintf(stderr, "%s\n", strerror(error_number));
+	}
+	fprintf(stderr, "%s: ", program);
+	fprintf(stderr, x0, x1);
+	fprintf(stderr, "\n");
+}
+
+void
 showargs(argv)
 	char	**argv;
 {
@@ -569,10 +648,9 @@ showargs(argv)
 	fprintf(stderr, "\n");
 }
 
-#define TmplDef "#define IMAKE_TEMPLATE"
-#define ImakeDef "#define INCLUDE_IMAKEFILE"
-#define TmplInc "#include IMAKE_TEMPLATE"
+#define ImakefileCHeader "/* imake - temporary file */"
 
+void
 CheckImakefileC(masterc)
 	char *masterc;
 {
@@ -584,12 +662,8 @@ CheckImakefileC(masterc)
 		if (inFile == NULL)
 			LogFatal("Refuse to overwrite: %s", masterc);
 		if ((fgets(mkcbuf, sizeof(mkcbuf), inFile) &&
-		     strncmp(mkcbuf, TmplDef, sizeof(TmplDef)-1)) ||
-		    (fgets(mkcbuf, sizeof(mkcbuf), inFile) &&
-		     strncmp(mkcbuf, ImakeDef, sizeof(ImakeDef)-1)) ||
-		    (fgets(mkcbuf, sizeof(mkcbuf), inFile) &&
-		     strncmp(mkcbuf, TmplInc, sizeof(TmplInc)-1)) ||
-		    fgets(mkcbuf, sizeof(mkcbuf), inFile))
+		     strncmp(mkcbuf, ImakefileCHeader, 
+			     sizeof(ImakefileCHeader)-1)))
 		{
 			fclose(inFile);
 			LogFatal("Refuse to overwrite: %s", masterc);
@@ -598,72 +672,45 @@ CheckImakefileC(masterc)
 	}
 }
 
-cppit(imakefile, template, masterc, outfd, outfname)
-	char	*imakefile;
-	char	*template;
-	char	*masterc;
-	FILE	*outfd;
-	char	*outfname;
-{
-	FILE	*inFile;
-	int	pid;
-	waitType	status;
+#define LocalDefineFmt	"#define %s \"%s\"\n"
+#define IncludeFmt	"#include %s\n"
+#define ImakeDefSym	"INCLUDE_IMAKEFILE"
+#define ImakeTmplSym	"IMAKE_TEMPLATE"
+#define OverrideWarning	"Warning: local file \"%s\" overrides global macros."
 
-	haveImakefileC = TRUE;
-	inFile = fopen(masterc, "w");
-	if (inFile == NULL)
-		LogFatal("Cannot open %s for output.", masterc);
-	if (fprintf(inFile, "%s \"%s\"\n", TmplDef, template) < 0 ||
-	    fprintf(inFile, "%s <%s>\n", ImakeDef, imakefile) < 0 ||
-	    fprintf(inFile, "%s\n", TmplInc) < 0 ||
-	    fclose(inFile))
-	    LogFatal("Cannot write to %s.", masterc);
-	/*
-	 * Fork and exec cpp
-	 */
-#ifdef WIN32
-	dup2(fileno(outfd), 1);
-	status = _spawnvp(_P_WAIT, cpp, cpp_argv);
-	if (status < 0)
-		LogFatal("Cannot spawn.", "");
-	if (status > 0)
-		LogFatalI("Exit code %d.", status);
-#else
-	pid = fork();
-	if (pid < 0)
-		LogFatal("Cannot fork.", "");
-	if (pid) {	/* parent */
-		while (wait(&status) > 0) {
-			errno = 0;
-			if (WIFSIGNALED(status))
-				LogFatalI("Signal %d.", waitSig(status));
-			if (WIFEXITED(status) && waitCode(status))
-				LogFatalI("Exit code %d.", waitCode(status));
-		}
+boolean
+optional_include(inFile, defsym, fname)
+        FILE	*inFile;
+        char    *defsym;
+        char    *fname;
+{
+	errno = 0;
+	if (access(fname, R_OK) == 0) {
+		LogMsg(OverrideWarning, fname);
+		return (fprintf(inFile, LocalDefineFmt, defsym, fname) < 0 ||
+			fprintf(inFile, IncludeFmt, defsym) < 0);
 	}
-	else {	/* child... dup and exec cpp */
-		if (verbose)
-			showargs(cpp_argv);
-		dup2(fileno(outfd), 1);
-		execvp(cpp, cpp_argv);
-		LogFatal("Cannot exec %s.", cpp);
-	}
-#endif
-	CleanCppOutput(outfd, outfname);
+	return FALSE;
 }
 
-makeit()
+void
+doit(outfd, cmd, argv)
+	FILE	*outfd;
+	char    *cmd;
+	char	**argv;
 {
 	int	pid;
 	waitType	status;
 
 	/*
-	 * Fork and exec make
+	 * Fork and exec the command.
 	 */
 #ifdef WIN32
-	status = _spawnvp(_P_WAIT, make_argv[0], make_argv);
+	if (outfd)
+		dup2(fileno(outfd), 1);
+	status = _spawnvp(_P_WAIT, cmd, argv);
 	if (status < 0)
-		LogFatal("Cannot spawn %s.", make_argv[0]);
+		LogFatal("Cannot spawn %s.", cmd);
 	if (status > 0)
 		LogFatalI("Exit code %d.", status);
 #else
@@ -678,16 +725,185 @@ makeit()
 			if (WIFEXITED(status) && waitCode(status))
 				LogFatalI("Exit code %d.", waitCode(status));
 		}
-	} else {	/* child... dup and exec cpp */
+	}
+	else {	/* child... dup and exec cmd */
 		if (verbose)
-			showargs(make_argv);
-		execvp(make_argv[0], make_argv);
-		LogFatal("Cannot exec %s.", make_argv[0]);
+			showargs(argv);
+		if (outfd)
+			dup2(fileno(outfd), 1);
+		execvp(cmd, argv);
+		LogFatal("Cannot exec %s.", cmd);
 	}
 #endif
 }
 
-char *CleanCppInput(imakefile)
+#ifndef WIN32
+static void
+parse_utsname(name, fmt, result, msg)
+     struct utsname *name;
+     char *fmt;
+     char *result;
+     char *msg;
+{
+  char buf[SYS_NMLN * 5 + 1];
+  char *ptr = buf;
+  int arg;
+
+  /* Assemble all the pieces into a buffer. */
+  for (arg = 0; fmt[arg] != ' '; arg++)
+    {
+      /* Our buffer is only guaranteed to hold 5 arguments. */
+      if (arg >= 5)
+	LogFatal(msg, fmt);
+
+      switch (fmt[arg])
+	{
+	case 's':
+	  if (arg > 0)
+	    *ptr++ = ' ';
+	  strcpy(ptr, name->sysname);
+	  ptr += strlen(ptr);
+	  break;
+
+	case 'n':
+	  if (arg > 0)
+	    *ptr++ = ' ';
+	  strcpy(ptr, name->nodename);
+	  ptr += strlen(ptr);
+	  break;
+
+	case 'r':
+	  if (arg > 0)
+	    *ptr++ = ' ';
+	  strcpy(ptr, name->release);
+	  ptr += strlen(ptr);
+	  break;
+
+	case 'v':
+	  if (arg > 0)
+	    *ptr++ = ' ';
+	  strcpy(ptr, name->version);
+	  ptr += strlen(ptr);
+	  break;
+
+	case 'm':
+	  if (arg > 0)
+	    *ptr++ = ' ';
+	  strcpy(ptr, name->machine);
+	  ptr += strlen(ptr);
+	  break;
+
+	default:
+	  LogFatal(msg, fmt);
+	}
+    }
+
+  /* Just in case... */
+  if (strlen(buf) >= sizeof(buf))
+    LogFatal("Buffer overflow parsing uname.", "");
+
+  /* Parse the buffer.  The sscanf() return value is rarely correct. */
+  *result = '\0';
+  (void) sscanf(buf, fmt + arg + 1, result);
+}
+#endif
+
+boolean
+define_os_defaults(inFile)
+	FILE	*inFile;
+{
+#ifndef WIN32
+#if (defined(DEFAULT_OS_NAME) || defined(DEFAULT_OS_MAJOR_REV) || \
+     defined(DEFAULT_OS_MINOR_REV) || defined(DEFAUL_OS_TEENY_REV))
+	struct utsname name;
+	char buf[SYS_NMLN * 5 + 1];
+
+	/* Obtain the system information. */
+	if (uname(&name) < 0)
+		LogFatal("Cannot invoke uname", "");
+
+# ifdef DEFAULT_OS_NAME
+	parse_utsname(&name, DEFAULT_OS_NAME, buf, 
+		      "Bad DEFAULT_OS_NAME syntax %s");
+#  ifdef DEFAULT_OS_NAME_FROB
+	DEFAULT_OS_NAME_FROB(buf, sizeof buf);
+#  endif
+	if (buf[0] != '\0')
+		fprintf(inFile, "#define DefaultOSName %s\n", buf);
+# endif
+
+# ifdef DEFAULT_OS_MAJOR_REV
+	parse_utsname(&name, DEFAULT_OS_MAJOR_REV, buf,
+		      "Bad DEFAULT_OS_MAJOR_REV syntax %s");
+#  ifdef DEFAULT_OS_MAJOR_REV_FROB
+	DEFAULT_OS_MAJOR_REV_FROB(buf, sizeof buf);
+#  endif
+	fprintf(inFile, "#define DefaultOSMajorVersion %s\n", *buf ? buf : "0");
+# endif
+
+# ifdef DEFAULT_OS_MINOR_REV
+	parse_utsname(&name, DEFAULT_OS_MINOR_REV, buf,
+		      "Bad DEFAULT_OS_MINOR_REV syntax %s");
+#  ifdef DEFAULT_OS_MINOR_REV_FROB
+	DEFAULT_OS_MINOR_REV_FROB(buf, sizeof buf);
+#  endif
+	fprintf(inFile, "#define DefaultOSMinorVersion %s\n", *buf ? buf : "0");
+# endif
+
+# ifdef DEFAULT_OS_TEENY_REV
+	parse_utsname(&name, DEFAULT_OS_TEENY_REV, buf,
+		      "Bad DEFAULT_OS_TEENY_REV syntax %s");
+#  ifdef DEFAULT_OS_TEENY_REV_FROB
+	DEFAULT_OS_TEENY_REV_FROB(buf, sizeof buf);
+#  endif
+	fprintf(inFile, "#define DefaultOSTeenyVersion %s\n", *buf ? buf : "0");
+# endif
+#endif
+#endif /* WIN32 */
+	return FALSE;
+}
+
+void
+cppit(imakefile, template, masterc, outfd, outfname)
+	char	*imakefile;
+	char	*template;
+	char	*masterc;
+	FILE	*outfd;
+	char	*outfname;
+{
+	FILE	*inFile;
+
+	haveImakefileC = TRUE;
+	inFile = fopen(masterc, "w");
+	if (inFile == NULL)
+		LogFatal("Cannot open %s for output.", masterc);
+	if (fprintf(inFile, "%s\n", ImakefileCHeader) < 0 ||
+	    define_os_defaults(inFile) ||
+	    optional_include(inFile, "IMAKE_LOCAL_DEFINES", "localdefines") ||
+	    optional_include(inFile, "IMAKE_ADMIN_DEFINES", "admindefines") ||
+	    fprintf(inFile, "#define %s <%s>\n", ImakeDefSym, imakefile) < 0 ||
+	    fprintf(inFile, LocalDefineFmt, ImakeTmplSym, template) < 0 ||
+	    fprintf(inFile, IncludeFmt, ImakeTmplSym) < 0 ||
+	    optional_include(inFile, "IMAKE_ADMIN_MACROS", "adminmacros") ||
+	    optional_include(inFile, "IMAKE_LOCAL_MACROS", "localmacros") ||
+	    fflush(inFile) || 
+	    fclose(inFile))
+		LogFatal("Cannot write to %s.", masterc);
+	/*
+	 * Fork and exec cpp
+	 */
+	doit(outfd, cpp, cpp_argv);
+	CleanCppOutput(outfd, outfname);
+}
+
+void
+makeit()
+{
+	doit(NULL, make_argv[0], make_argv);
+}
+
+char *
+CleanCppInput(imakefile)
 	char	*imakefile;
 {
 	FILE	*outFile = NULL;
@@ -746,7 +962,7 @@ char *CleanCppInput(imakefile)
 			(void) mktemp(tmpImakefile);
 			outFile = fopen(tmpImakefile, "w");
 			if (outFile == NULL)
-			    LogFatal("Cannot open %s for write.\n",
+			    LogFatal("Cannot open %s for write.",
 				tmpImakefile);
 		    }
 		    writetmpfile(outFile, punwritten, pbuf-punwritten,
@@ -770,6 +986,7 @@ char *CleanCppInput(imakefile)
 	return(imakefile);
 }
 
+void
 CleanCppOutput(tmpfd, tmpfname)
 	FILE	*tmpfd;
 	char	*tmpfname;
@@ -805,6 +1022,7 @@ CleanCppOutput(tmpfd, tmpfname)
  * space from the end of the line.  Cpp magic cookies are also thrown away.
  * "XCOMM" token is transformed to "#".
  */
+boolean
 isempty(line)
 	register char	*line;
 {
@@ -878,7 +1096,8 @@ isempty(line)
 }
 
 /*ARGSUSED*/
-char *ReadLine(tmpfd, tmpfname)
+char *
+ReadLine(tmpfd, tmpfname)
 	FILE	*tmpfd;
 	char	*tmpfname;
 {
@@ -887,6 +1106,9 @@ char *ReadLine(tmpfd, tmpfname)
 	register char	*p1, *p2;
 
 	if (! initialized) {
+#ifdef WIN32
+		FILE *fp = tmpfd;
+#endif
 		int	total_red;
 		struct stat	st;
 
@@ -895,23 +1117,29 @@ char *ReadLine(tmpfd, tmpfname)
 		 */
 		fseek(tmpfd, 0, 0);
 		if (fstat(fileno(tmpfd), &st) < 0)
-			LogFatal("cannot stat %s for size\n", tmpMakefile);
+			LogFatal("cannot stat %s for size", tmpMakefile);
 		pline = buf = Emalloc((int)st.st_size+1);
 		total_red = fread(buf, 1, st.st_size, tmpfd);
 		if (total_red == 0  &&  st.st_size != 0)
-			LogFatal("cannot read %s\n", tmpMakefile);
+			LogFatal("cannot read %s", tmpMakefile);
 		end = buf + total_red;
 		*end = '\0';
 		fseek(tmpfd, 0, 0);
 #if defined(SYSV) || defined(WIN32) || defined(USE_FREOPEN)
-		freopen(tmpfname, "w+", tmpfd);
+		tmpfd = freopen(tmpfname, "w+", tmpfd);
+#ifdef WIN32
+		if (! tmpfd) /* if failed try again */
+			tmpfd = freopen(tmpfname, "w+", fp);
+#endif
+		if (! tmpfd)
+			LogFatal("cannot reopen %s\n", tmpfname);
 #else	/* !SYSV */
 		ftruncate(fileno(tmpfd), (off_t) 0);
 #endif	/* !SYSV */
 		initialized = TRUE;
 	    fprintf (tmpfd, "# Makefile generated by imake - do not edit!\n");
 	    fprintf (tmpfd, "# %s\n",
-		"$XConsortium: imake.c /main/88 1995/12/08 16:33:13 gildea $");
+		"$XConsortium: imake.c /main/90 1996/11/13 14:43:23 lehors $");
 	}
 
 	for (p1 = pline; p1 < end; p1++) {
@@ -941,23 +1169,25 @@ char *ReadLine(tmpfd, tmpfname)
 	return(p2);
 }
 
+void
 writetmpfile(fd, buf, cnt, fname)
 	FILE	*fd;
 	int	cnt;
 	char	*buf;
 	char	*fname;
 {
-	if (fwrite(buf, sizeof(char), cnt, fd) != cnt)
+	if (fwrite(buf, sizeof(char), cnt, fd) == -1)
 		LogFatal("Cannot write to %s.", fname);
 }
 
-char *Emalloc(size)
+char *
+Emalloc(size)
 	int	size;
 {
 	char	*p;
 
 	if ((p = malloc(size)) == NULL)
-		LogFatalI("Cannot allocate %d bytes\n", size);
+		LogFatalI("Cannot allocate %d bytes", size);
 	return(p);
 }
 
@@ -976,13 +1206,13 @@ KludgeOutputLine(pline)
 	    	break;
 	    case ' ':	/*May need a tab*/
 	    default:
-#ifdef INLINE_SYNTAX
+# ifdef INLINE_SYNTAX
 		if (*p == '<' && p[1] == '<') { /* inline file close */
 		    InInline--;
 		    InRule = TRUE;
 		    break;
 		}
-#endif
+# endif
 		/*
 		 * The following cases should not be treated as beginning of 
 		 * rules:
@@ -994,9 +1224,9 @@ KludgeOutputLine(pline)
 		    if (quotechar) {
 			if (quotechar == '\\' ||
 			    (*p == quotechar &&
-#ifdef WIN32
+# ifdef WIN32
 			     quotechar != ')' &&
-#endif
+# endif
 			     p[-1] != '\\'))
 			    quotechar = '\0';
 			continue;
@@ -1017,19 +1247,19 @@ KludgeOutputLine(pline)
 			quotechar = ']';
 			break;
 		    case '=':
-#ifdef REMOVE_CPP_LEADSPACE
+# ifdef REMOVE_CPP_LEADSPACE
 			if (!InRule && **pline == ' ') {
 			    while (**pline == ' ')
 				(*pline)++;
 			}
-#endif
+# endif
 			goto breakfor;
-#ifdef INLINE_SYNTAX
+# ifdef INLINE_SYNTAX
 		    case '<':
 			if (p[1] == '<') /* inline file start */
 			    InInline++;
 			break;
-#endif
+# endif
 		    case ':':
 			if (p[1] == '=')
 			    goto breakfor;
@@ -1053,7 +1283,8 @@ KludgeResetRule()
 }
 #endif /* FIXUP_CPP_WHITESPACE */
 
-char *Strdup(cp)
+char *
+Strdup(cp)
 	register char *cp;
 {
 	register char *new = Emalloc(strlen(cp) + 1);
