@@ -1,5 +1,4 @@
-/* $XConsortium: Xrm.c,v 1.89 95/06/08 23:20:39 gildea Exp $ */
-/* $XFree86: xc/lib/X11/Xrm.c,v 3.3 1996/05/10 06:55:08 dawes Exp $ */
+/* $TOG: Xrm.c /main/91 1997/05/21 14:09:34 kaleb $ */
 
 /***********************************************************
 Copyright 1987, 1988, 1990 by Digital Equipment Corporation, Maynard
@@ -52,6 +51,7 @@ other dealings in this Software without prior written authorization
 from the X Consortium.
 
 */
+/* $XFree86: xc/lib/X11/Xrm.c,v 3.3.4.2 1997/05/22 14:00:26 dawes Exp $ */
 
 #include	<stdio.h>
 #include	<ctype.h>
@@ -1070,8 +1070,18 @@ void XrmQPutStringResource(pdb, bindings, quarks, str)
  * Chris Peterson, MIT X Consortium		5/17/90.
  */
 
-#define LIST_SIZE 101
-#define BUFFER_SIZE 100
+/* 
+ * Xlib spec says max 100 quarks in a lookup, will stop and return if
+ * return if any single production's lhs has more than 100 components.
+ */
+#define QLIST_SIZE 100
+
+/* 
+ * This should be big enough to handle things like the XKeysymDB or biggish
+ * ~/.Xdefaults or app-defaults files. Anything bigger will be allocated on
+ * the heap.
+ */
+#define DEF_BUFF_SIZE 8192
 
 static void GetIncludeFile();
 
@@ -1081,19 +1091,21 @@ static void GetDatabase(db, str, filename, doall)
     char *filename;
     Bool doall;
 {
+    char *rhs;
+    char *lhs, lhs_s[DEF_BUFF_SIZE];
+    XrmQuark quarks[QLIST_SIZE + 1];	/* allow for a terminal NullQuark */
+    XrmBinding bindings[QLIST_SIZE + 1];
+
     register char *ptr;
     register XrmBits bits = 0;
     register char c;
-    int len;
     register Signature sig;
     register char *ptr_max;
-    register XrmQuarkList t_quarks;
+    register int num_quarks;
     register XrmBindingList t_bindings;
 
-    int alloc_chars = BUFSIZ;
-    char buffer[BUFSIZ], *value_str;
-    XrmQuark quarks[LIST_SIZE];
-    XrmBinding bindings[LIST_SIZE];
+    int len, alloc_chars;
+    unsigned long str_len;
     XrmValue value;
     Bool only_pcs;
     Bool dolines;
@@ -1101,9 +1113,33 @@ static void GetDatabase(db, str, filename, doall)
     if (!db)
 	return;
 
-    if (!(value_str = Xmalloc(sizeof(char) * alloc_chars)))
+    /* 
+     * if strlen (str) < DEF_BUFF_SIZE allocate buffers on the stack for 
+     * speed otherwise malloc the buffer. From a buffer overflow standpoint
+     * we can be sure that neither: a) a component on the lhs, or b) a
+     * value on the rhs, will be longer than the overall length of str,
+     * i.e. strlen(str).
+     *
+     * This should give good performance when parsing "*foo: bar" type 
+     * databases as might be passed with -xrm command line options; but
+     * with larger databases, e.g. .Xdefaults, app-defaults, or KeysymDB
+     * files, the size of the buffers will be overly large. One way
+     * around this would be to double-parse each production with a resulting
+     * performance hit. In any event we can be assured that a lhs component
+     * name or a rhs value won't be longer than str itself.
+     */
+
+    str_len = strlen (str);
+    if (DEF_BUFF_SIZE > str_len) lhs = lhs_s;
+    else if ((lhs = (char*) Xmalloc (str_len)) == NULL) 
 	return;
 
+    alloc_chars = DEF_BUFF_SIZE < str_len ? str_len : DEF_BUFF_SIZE;
+    if ((rhs = (char*) Xmalloc (alloc_chars)) == NULL) {
+	if (lhs != lhs_s) Xfree (lhs);
+	return;
+    }
+
     (*db->methods->mbinit)(db->mbstate);
     str--;
     dolines = True;
@@ -1182,19 +1218,13 @@ static void GetDatabase(db, str, filename, doall)
 	/*
 	 * Third: loop through the LHS of the resource specification
 	 * storing characters and converting this to a Quark.
-	 *
-	 * If the number of quarks is greater than LIST_SIZE - 1.  This
-	 * function will trash your memory.
-	 *
-	 * If the length of any quark is larger than BUFSIZ this function
-	 * will also trash memory.
 	 */
 	
+	num_quarks = 0;
 	t_bindings = bindings;
-	t_quarks = quarks;
 
 	sig = 0;
-	ptr = buffer;
+	ptr = lhs;
 	*t_bindings = XrmBindTightly;	
 	for(;;) {
 	    if (!is_binding(bits)) {
@@ -1204,8 +1234,15 @@ static void GetDatabase(db, str, filename, doall)
 		    bits = next_char(c, str);
 		}
 
-		*t_quarks++ = _XrmInternalStringToQuark(buffer, ptr - buffer,
-							sig, False);
+		quarks[num_quarks++] = 
+			_XrmInternalStringToQuark(lhs, ptr - lhs, sig, False);
+
+		if (num_quarks > QLIST_SIZE) {
+		    Xfree(rhs);
+		    if (lhs != lhs_s) Xfree (lhs);
+		    (*db->methods->mbfinish)(db->mbstate);
+		    return;
+		}
 
 		if (is_separator(bits))  {
 		    if (!is_space(bits))
@@ -1224,7 +1261,7 @@ static void GetDatabase(db, str, filename, doall)
 
 		    if (is_separator(bits))
 			break;
-		    t_quarks--;
+		    num_quarks--;
 		    continue;
 		}
 
@@ -1234,7 +1271,7 @@ static void GetDatabase(db, str, filename, doall)
 		    *(++t_bindings) = XrmBindLoosely;
 
 		sig = 0;
-		ptr = buffer;
+		ptr = lhs;
 	    }
 	    else {
 		/*
@@ -1254,7 +1291,7 @@ static void GetDatabase(db, str, filename, doall)
 	    bits = next_char(c, str);
 	}
 
-	*t_quarks = NULLQUARK;
+	quarks[num_quarks] = NULLQUARK;
 
 	/*
 	 * Make sure that there is a ':' in this line.
@@ -1310,7 +1347,7 @@ static void GetDatabase(db, str, filename, doall)
 	 * Fifth: Process the right hand side.
 	 */
 
-	ptr = value_str;
+	ptr = rhs;
 	ptr_max = ptr + alloc_chars - 4;
 	only_pcs = True;
 	len = 1;
@@ -1437,17 +1474,18 @@ static void GetDatabase(db, str, filename, doall)
 		char * temp_str;
 
 		alloc_chars += BUFSIZ/10;		
-		temp_str = Xrealloc(value_str, sizeof(char) * alloc_chars);
+		temp_str = Xrealloc(rhs, sizeof(char) * alloc_chars);
 
 		if (!temp_str) {
-		    Xfree(value_str);
+		    Xfree(rhs);
+		    if (lhs != lhs_s) Xfree (lhs);
 		    (*db->methods->mbfinish)(db->mbstate);
 		    return;
 		}
 
-		ptr = temp_str + (ptr - value_str); /* reset pointer. */
-		value_str = temp_str;
-		ptr_max = value_str + alloc_chars - 4;
+		ptr = temp_str + (ptr - rhs); /* reset pointer. */
+		rhs = temp_str;
+		ptr_max = rhs + alloc_chars - 4;
 	    }
 	}
 
@@ -1459,13 +1497,15 @@ static void GetDatabase(db, str, filename, doall)
 	*ptr++ = '\0';
 
 	/* Store it in database */
-	value.size = ptr - value_str;
-	value.addr = (XPointer) value_str;
+	value.size = ptr - rhs;
+	value.addr = (XPointer) rhs;
 	
 	PutEntry(db, bindings, quarks, XrmQString, &value);
     }
 
-    Xfree(value_str);
+    if (lhs != lhs_s) Xfree (lhs);
+    Xfree (rhs);
+
     (*db->methods->mbfinish)(db->mbstate);
 }
 
