@@ -1,5 +1,5 @@
 /* $XConsortium: Xtrans.c,v 1.31 95/03/28 19:49:02 mor Exp $ */
-/* $XFree86: xc/lib/xtrans/Xtrans.c,v 3.13 1996/10/03 08:29:46 dawes Exp $ */
+/* $XFree86: xc/lib/xtrans/Xtrans.c,v 3.15 1996/12/09 11:50:25 dawes Exp $ */
 /*
 
 Copyright (c) 1993, 1994  X Consortium
@@ -785,6 +785,23 @@ char		*port;
     return ciptr->transptr->CreateListener (ciptr, port);
 }
 
+int
+TRANS(NoListen) (char * protocol)
+	
+{
+   Xtransport *trans;
+   
+   if ((trans = TRANS(SelectTransport)(protocol)) == NULL) 
+   {
+	PRMSG (1,"TRANS(TransNoListen): unable to find transport: %s\n", 
+	       protocol, 0, 0);
+
+	return -1;
+   }
+   
+   trans->flags |= TRANS_NOLISTEN;
+   return 0;
+}
 
 int
 TRANS(ResetListener) (ciptr)
@@ -1050,7 +1067,8 @@ complete_network_count ()
 
     for (i = 0; i < NUMTRANS; i++)
     {
-	if (Xtransports[i].transport->flags & TRANS_ALIAS)
+	if (Xtransports[i].transport->flags & TRANS_ALIAS
+   	 || Xtransports[i].transport->flags & TRANS_NOLISTEN)
 	    continue;
 
 	if (Xtransports[i].transport->flags & TRANS_LOCAL)
@@ -1087,7 +1105,7 @@ XtransConnInfo 	**ciptrs_ret;
     {
 	Xtransport *trans = Xtransports[i].transport;
 
-	if (trans->flags&TRANS_ALIAS)
+	if (trans->flags&TRANS_ALIAS || trans->flags&TRANS_NOLISTEN)
 	    continue;
 
 	sprintf(buffer,"%s/:%s", trans->TransName, port ? port : "");
@@ -1193,7 +1211,7 @@ XtransConnInfo 	**ciptrs_ret;
     {
 	Xtransport *trans = Xtransports[i].transport;
 
-	if (trans->flags&TRANS_ALIAS)
+	if (trans->flags&TRANS_ALIAS || trans->flags&TRANS_NOLISTEN)
 	    continue;
 
 	sprintf(buffer,"%s/:%s", trans->TransName, port ? port : "");
@@ -1328,7 +1346,7 @@ int 		iovcnt;
 
 #endif /* CRAY */
 
-#if (defined(SYSV) && defined(i386)) || defined(WIN32) || defined(__sxg__) || defined(__EMX__)
+#if (defined(SYSV) && defined(i386) && !defined(SCO325)) || defined(WIN32) || defined(__sxg__) || defined(__EMX__)
 
 /*
  * emulate readv
@@ -1364,7 +1382,7 @@ int 		iovcnt;
 
 #endif /* SYSV && i386 || WIN32 || __sxg__ */
 
-#if defined(WIN32) || defined(__sxg__) || defined(__EMX__)
+#if (defined(SYSV) && defined(i386) && !defined(SCO325)) || defined(WIN32) || defined(__sxg__) || defined(__EMX__)
 
 /*
  * emulate writev
@@ -1398,7 +1416,7 @@ int 		iovcnt;
     return total;
 }
 
-#endif /* WIN32 || __sxg__ */
+#endif /* SYSV && i386 || WIN32 || __sxg__ */
 
 
 #if (defined(_POSIX_SOURCE) && !defined(AIXV3)) || defined(hpux) || defined(USG) || defined(SVR4)
