@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/i128/i128init.c,v 3.4 1996/08/13 11:29:41 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/i128/i128init.c,v 3.6.2.1 1997/05/23 12:19:38 dawes Exp $ */
 /*
  * Copyright 1995 by Robin Cutshaw <robin@XFree86.Org>
  *
@@ -21,7 +21,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $XConsortium: i128init.c /main/2 1995/12/17 08:13:10 kaleb $ */
+/* $XConsortium: i128init.c /main/5 1996/10/19 17:52:17 kaleb $ */
 
 
 #include "i128.h"
@@ -31,8 +31,8 @@
 
 
 typedef struct {
-unsigned long i128_base_g[0x60/4];	/* base g registers                  */
-unsigned long i128_base_w[0x28/4];	/* base w registers                  */
+CARD32 i128_base_g[0x60/4];	/* base g registers                  */
+CARD32 i128_base_w[0x28/4];	/* base w registers                  */
 unsigned char Ti302X[0x40];		/* Ti302[05] registers               */
 unsigned char Ti3025[9];		/* Ti3025 N,M,P for PCLK, MCLK, LCLK */
 unsigned char IBMRGB[0x101];		/* IBMRGB registers                  */
@@ -132,6 +132,8 @@ saveI128state()
 		iR.Ti302X[TI_GENERAL_IO_DATA] = i128mem.rbase_g_b[DATA_TI];
 		i128mem.rbase_g_b[INDEX_TI] = TI_MCLK_DCLK_CONTROL;
 		iR.Ti302X[TI_MCLK_DCLK_CONTROL] = i128mem.rbase_g_b[DATA_TI];
+		i128mem.rbase_g_b[INDEX_TI] = TI_COLOR_KEY_CONTROL;
+		iR.Ti302X[TI_COLOR_KEY_CONTROL] = i128mem.rbase_g_b[DATA_TI];
 
 		i128mem.rbase_g_b[INDEX_TI] = TI_PLL_CONTROL;
 		i128mem.rbase_g_b[DATA_TI] = 0x00;
@@ -272,6 +274,8 @@ restoreI128state()
 		i128mem.rbase_g_b[DATA_TI] = iR.Ti302X[TI_GENERAL_IO_DATA];
 		i128mem.rbase_g_b[INDEX_TI] = TI_MCLK_DCLK_CONTROL;
 		i128mem.rbase_g_b[DATA_TI] = iR.Ti302X[TI_MCLK_DCLK_CONTROL];
+		i128mem.rbase_g_b[INDEX_TI] = TI_COLOR_KEY_CONTROL;
+		i128mem.rbase_g_b[DATA_TI] = iR.Ti302X[TI_COLOR_KEY_CONTROL];
 	} else if ((i128RamdacType == IBM526_DAC) ||
 		   (i128RamdacType == IBM528_DAC)) {
 		short i;
