@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86_Mouse.c,v 3.19 1996/10/16 14:40:51 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86_Mouse.c,v 3.21.2.2 1997/05/12 12:52:30 hohndel Exp $ */
 /*
  *
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
@@ -24,12 +24,16 @@
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $XConsortium: xf86_Mouse.c /main/10 1996/01/30 15:16:12 kaleb $ */
+/* $XConsortium: xf86_Mouse.c /main/21 1996/10/27 11:05:32 kaleb $ */
 
 /*
  * [JCH-96/01/21] Added fourth button support for P_GLIDEPOINT mouse protocol.
  */
 
+/*
+ * [TVO-97/03/05] Added microsoft IntelliMouse support
+ */
+
 #define NEED_EVENTS
 #include "X.h"
 #include "Xproto.h"
@@ -95,6 +99,7 @@ Bool xf86SupportedMouseTypes[] =
 	TRUE,	/* PS/2 */
 	TRUE,	/* Hitachi Tablet */
 	TRUE,	/* ALPS GlidePoint */
+	TRUE,   /* Microsoft IntelliMouse */
 };
 
 int xf86NumMouseTypes = sizeof(xf86SupportedMouseTypes) /
@@ -120,6 +125,7 @@ unsigned short xf86MouseCflags[] =
 	0,						     /* PS/2 */
 	(CS8                   | CREAD | CLOCAL | HUPCL ),   /* mmhitablet */
 	(CS7                   | CREAD | CLOCAL | HUPCL ),   /* GlidePoint */
+	(CS7                   | CREAD | CLOCAL | HUPCL ),   /* IntelliMouse */
 };
 #endif /* ! MOUSE_PROTOCOL_IN_KERNEL */
 
@@ -293,7 +299,7 @@ xf86MouseProtocol(device, rBuf, nBytes)
   static unsigned char pBuf[8];
   MouseDevPtr          mouse = MOUSE_DEV(device);
   
-  static unsigned char proto[9][5] = {
+  static unsigned char proto[10][5] = {
     /*  hd_mask hd_id   dp_mask dp_id   nobytes */
     { 	0x40,	0x40,	0x40,	0x00,	3 	},  /* MicroSoft */
     {	0xf8,	0x80,	0x00,	0x00,	5	},  /* MouseSystems */
@@ -305,6 +311,7 @@ xf86MouseProtocol(device, rBuf, nBytes)
     {	0xc0,	0x00,	0x00,	0x00,	3	},  /* PS/2 mouse */
     {	0xe0,	0x80,	0x80,	0x00,	3	},  /* MM_HitTablet */
     { 	0x40,	0x40,	0x40,	0x00,	3 	},  /* GlidePoint */
+    { 	0x40,	0x40,	0x40,	0x00,	4 	}   /* IntelliMouse */
   };
   
   for ( i=0; i < nBytes; i++) {
@@ -392,7 +399,6 @@ xf86MouseProtocol(device, rBuf, nBytes)
 
     pBuf[pBufP++] = rBuf[i];
     if (pBufP != proto[mouse->mseType][4]) continue;
-
     /*
      * assembly full package
      */
@@ -460,11 +466,39 @@ xf86MouseProtocol(device, rBuf, nBytes)
       dy = (pBuf[0] & 0x20) ?  -(pBuf[2]-256) : -pBuf[2];
       break;
 #endif
+    case P_MSINTELLIMOUSE:              /* Microsoft IntelliMouse */
+      dx = (char) ((pBuf[0] & 0x03) << 6 | pBuf[1]);
+      dy = (char) ((pBuf[0] & 0x0c) << 4 | pBuf[2]);
+      buttons = 
+	((pBuf[0] & 0x10) ? 1 : 0) |
+	((pBuf[3] & 0x10) ? 2 : 0) |
+	((pBuf[0] & 0x20) ? 4 : 0) |
+	((pBuf[3] & 0x08) ? 8 : 0) |
+	(((!(pBuf[3] & 0x08)) && (pBuf[3] & 0x07)) ? 16 : 0);
+      /*
+       * {
+       *   static int seqno = 0;
+       *   fprintf (stderr, "%d - dx=%4d dy=%4d buttons=%c%c%c%c%c\n", 
+       *	    seqno++, dx, dy,
+       *	    (buttons &  1) ? 'R' : ' ',
+       *	    (buttons &  2) ? 'M' : ' ',
+       *	    (buttons &  4) ? 'L' : ' ',
+       *	    (buttons &  8) ? 'U' : ' ',
+       *	    (buttons & 16) ? 'D' : ' ');
+       * }
+       */
+      break;
     default: /* There's a table error */
 	continue;
     }
 
     xf86PostMseEvent(device, buttons, dx, dy);
+
+    if ((mouse->mseType == P_MSINTELLIMOUSE)  &&
+	(buttons & (8 + 16))) {
+      xf86PostMseEvent(device, buttons & ~ (8 + 16), 0, 0);
+    }
+
     pBufP = 0;
   }
 }
@@ -606,7 +640,8 @@ xf86MouseAllocate()
     local->atom = 0;
     local->dev = NULL;
     local->private = mouse;
-
+    local->always_core_feedback = 0;
+    
     mouse->device = NULL;
     mouse->mseFd = -1;
     mouse->mseDevice = "";
@@ -614,6 +649,7 @@ xf86MouseAllocate()
     mouse->baudRate = -1;
     mouse->oldBaudRate = -1;
     mouse->sampleRate = -1;
+    mouse->local = local;
     
 #ifdef EXTMOUSEDEBUG
     ErrorF("xf86MouseAllocate mouse=0x%x\n", local->private);
