@@ -1,10 +1,10 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/xf86Version.h,v 3.205 1996/10/26 09:33:28 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/xf86Version.h,v 3.236.2.26 1997/06/02 01:46:53 dawes Exp $ */
 
-#define XF86_VERSION " 3.2 "
+#define XF86_VERSION " 3.3 "
 
 /* The finer points in versions... */
 #define XF86_VERSION_MAJOR	3
-#define XF86_VERSION_MINOR	2
+#define XF86_VERSION_MINOR	3
 #define XF86_VERSION_SUBMINOR	0
 #define XF86_VERSION_BETA	0	/* 0="", 1="A", 2="B", etc... */
 #define XF86_VERSION_ALPHA	0	/* 0="", 1="a", 2="b", etc... */
@@ -18,6 +18,6 @@
 			XF86_VERSION_BETA,			\
 			XF86_VERSION_ALPHA)
 
-#define XF86_DATE	"Oct 26 1996"
+#define XF86_DATE	"Jun  2 1997"
 
-/* $XConsortium: xf86Version.h /main/36 1996/01/31 10:07:08 kaleb $ */
+/* $XConsortium: xf86Version.h /main/78 1996/10/28 05:42:10 kaleb $ */
